from visual import *
from visual.graph import *
from visual.controls import *
from PIL import ImageGrab

scene.title="Dynamic Equilibrium Simulation"
scene.range = 10
scene.autocenter = 1
scene.autoscale = 1

#image grab stuff
image = False
padding = 4
directory = "movie-images"

#demensions
rope_length = 5.0
spring_length = 1.0
spring_start = 4.0
size = 0.75
scale = 4

#constants
g = 4.0
k = 1.0
dt = 0.05
time = 40

#pulley and strings for visual purposes
pulley = cylinder(pos=(0,0,-0.25), axis=(0,0,0.5), radius=1)
left_string = cylinder(pos=(-1,0,0), axis=(0,-rope_length,0), radius=0.05)
right_string = cylinder(pos=(1,0,), axis=(0,-rope_length,0), radius=0.05)

#spring and boxes objects
box_a = box(pos = left_string.pos + left_string.axis, 
            length = size, width = size, height = size*2, color = color.blue)
box_b= box(pos = right_string.pos + right_string.axis,                            
            length = size, width = size, height = size, color = color.red)

spring = helix(pos = box_b.pos, axis = (0, -spring_start, 0), 
               radius = 0.25, coils = 10, thickness = 0.05) 
box_c = box(pos = spring.pos + spring.axis,
            length = size, width = size, height = size, color = color.yellow)

#initial conditions
t = 0.0
box_a.vel = 0.0
box_b.vel = 0.0
box_c.vel = 0.0

Fs = 0.0
T = 0.0

#force comparisons on the red box, scaled down
T_force = arrow(pos = box_b.pos + (2, 0, 0))
G_force = arrow(pos = box_b.pos + (3, 0, 0), color = color.red)
G_force.axis = (0,g/scale, 0)
S_force = arrow(pos = G_force.pos + G_force.axis, color = color.yellow)

#graph of speeds of the red and blue box
gdisplay(title='Position', xtitle='Time', ytitle='Position', 
            xmax=time, ymax=0, ymin=-rope_length * 3, x=0, y=450, width=500, height=300)
line_a = gcurve(color=color.blue)
line_b = gcurve(color=color.red)
line_c = gcurve(color=color.yellow)  

scene.autocenter = 0
while t < time:
    #tells computer how fast to animate
    rate(50) 
    
    #calculating the forces
    dL = (spring_length - (box_b.pos - box_c.pos).y) 
    Fs = -k * dL
    T = 4.0 * g / 3.0 + 2.0 * Fs / 3.0
    
    #drawing force arrows, scaled
    S_force.axis = (0, Fs/scale, 0)
    T_force.axis = (0,T/scale,0)
    
    #for creating the animation
    if(image):
        im = ImageGrab.grab((40,50,500,520))
        timestamp = int(t/dt)
        im.save(directory + "%05d" % timestamp  +'.png')
    
    #velocity
    box_a.vel += dt * (T / 2.0  - g)
    box_b.vel += dt * (T - Fs - g)
    box_c.vel += dt *(-g + Fs)

    #position
    box_a.pos.y += dt * box_a.vel
    box_b.pos.y += dt * box_b.vel
    box_c.pos.y += dt * box_c.vel
    
    #springs and pulley ropes
    left_string.axis.y = box_a.pos.y
    right_string.axis.y = box_b.pos.y
    spring.pos.y = box_b.pos.y
    spring.axis = box_c.pos - box_b.pos                                             
    
    #plots of positions
    line_a.plot(pos=(t, box_a.y))
    line_b.plot(pos=(t, box_b.y))
    line_c.plot(pos=(t, box_c.y))
    
    t += dt
