from visual import *
from visual.graph import *
from PIL import ImageGrab
#graphics stuff
scene = display(x=0, y=0, width=500, height=500)

scene.title="Dynamic Equilibrium Simulation"
scene.range = 10
scene.autocenter = 1


#demensions
rope_length = 3.0
spring_length = 2.0
spring_start = 3.0
size = 0.75
scale = 5
#constants
g = 5.0
k = 4.0
m = 1.0
dt = 0.0125

time = 20.0

box_b= box(pos = (0,2,0),                            
            length = size, width = size, height = size, color = color.red)

spring = helix(pos = box_b.pos, axis = (0, -spring_start, 0), 
               radius = 0.25, coils = 10, thickness = 0.05) 
box_c = box(pos = spring.pos + spring.axis,
            length = size, width = size, height = size, color = color.yellow)

t = 0.0
box_b.vel = 0.0
box_c.vel = 0.0

Fs = 0.0
T = 0.0

#forces on red box
T_force = arrow(pos = box_b.pos + (2, 0, 0))
G_force = arrow(pos = box_b.pos + (2, 0, 0), color = color.red)
G_force.axis = (0,-g/scale, 0)
S_force = arrow(pos = G_force.pos + G_force.axis, color = color.yellow)

gdisplay(title='Velocity', xtitle='Time', ytitle='Velocity',
    xmax=20.0, ymax=5, ymin=-10, x=0, y=500, width=500, height=300)
vel_c = gcurve(color=color.yellow)
pos_c = gdots(color=color.yellow, size = 5)

#So scene doesn't bounce
scene.autocenter = 0

while t < time:
    rate(100)
    Fs = -k * (spring_length - (box_b.pos - box_c.pos).y)
    T = g + Fs
    
    S_force.axis = (0,-Fs/scale, 0)
    T_force.axis = (0, T/scale,0)
    
    im = ImageGrab.grab((40,50,500,820))
    im.save('spring/4/img-' + str(int(t/dt)) +'.png') 
    
    box_c.vel += dt *(-g + Fs)
    box_c.pos.y += dt * box_c.vel
    
    spring.pos.y = box_b.pos.y
    spring.axis = box_c.pos - box_b.pos                                             
    
    vel_c.plot(pos=(t, box_c.vel))
    pos_c.plot(pos=(t, box_c.y))

    t += dt
    
